(ns com.aurellem.gb-driver
  (:import com.aurellem.gb.Gb)
  (:import java.io.File)
  (:import (java.nio IntBuffer ByteOrder)))

(Gb/loadVBA)

(def yellow-rom-image
  (File. "/home/r/proj/pokemon-escape/roms/yellow.gbc"))

(def yellow-save-file
  (File. "/home/r/proj/pokemon-escape/roms/yellow.sav"))

(defn vba-init []
  (.delete yellow-save-file)
  (future
    (Gb/startEmulator (.getCanonicalPath yellow-rom-image))))

(defn cpu-data [size arr-fn]
  (let [store (int-array size)]
    (fn [] 
      (arr-fn store)
      store)))

(def ram
  (cpu-data (Gb/getRAMSize) #(Gb/getRAM %)))

(def rom 
  (cpu-data (Gb/getROMSize) #(Gb/getROM %)))

(def working-ram 
  (cpu-data Gb/WRAM_SIZE #(Gb/getWRAM %)))

(def video-ram 
  (cpu-data Gb/VRAM_SIZE #(Gb/getVRAM %)))

(def registers
  (cpu-data Gb/NUM_REGISTERS #(Gb/getRegisters %)))

(defn step
  ([] (Gb/step))
  ([mask] (Gb/step mask)))

(defn shutdown [] (Gb/shutdown))

   
  

